@@ -7,17 +7,18 @@ Configuration is controlled via environment variables:
 - OPENAPI_SPEC_URL_<hash>: Unique URL per test, falls back to OPENAPI_SPEC_URL.
 - TOOL_WHITELIST: Comma-separated list of allowed endpoint paths.
 - SERVER_URL_OVERRIDE: Optional override for the base URL from the OpenAPI spec.
-- API_KEY: Optional token for endpoints requiring authentication.
-- API_AUTH_TYPE: Optional type like 'Bearer' or 'Api-Key'.
+- API_KEY: Generic token for Bearer header.
+- STRIP_PARAM: Param name (e.g., "auth") to remove from parameters.
 """
 
 import os
 import sys
 import json
 import requests
-
+from typing import Dict, Any
+from mcp import types
 from mcp.server.fastmcp import FastMCP
-from mcp_openapi_proxy.utils import setup_logging, is_tool_whitelisted, fetch_openapi_spec, get_auth_headers, build_base_url, normalize_tool_name
+from mcp_openapi_proxy.utils import setup_logging, is_tool_whitelisted, fetch_openapi_spec, build_base_url, normalize_tool_name, handle_auth, strip_parameters, get_tool_prefix
 
 logger = setup_logging(debug=os.getenv("DEBUG", "").lower() in ("true", "1", "yes"))
 
@@ -28,6 +29,7 @@ mcp = FastMCP("OpenApiProxy-Fast")
 
 @mcp.tool()
 def list_functions(*, env_key: str = "OPENAPI_SPEC_URL") -> str:
+    """Lists available functions derived from the OpenAPI specification."""
     logger.debug("Executing list_functions tool.")
     spec_url = os.environ.get(env_key, os.environ.get("OPENAPI_SPEC_URL"))
     whitelist = os.getenv('TOOL_WHITELIST')
@@ -47,6 +49,7 @@ def list_functions(*, env_key: str = "OPENAPI_SPEC_URL") -> str:
         logger.debug("No paths found in spec.")
         return json.dumps([])
     functions = {}
+    prefix = get_tool_prefix()
     for path, path_item in paths.items():
         logger.debug(f"Processing path: {path}")
         if not path_item:
@@ -67,18 +70,38 @@ def list_functions(*, env_key: str = "OPENAPI_SPEC_URL") -> str:
                 continue
             raw_name = f"{method.upper()} {path}"
             function_name = normalize_tool_name(raw_name)
+            if prefix:
+                function_name = f"{prefix}{function_name}"
             if function_name in functions:
                 logger.debug(f"Skipping duplicate tool name: {function_name}")
                 continue
             function_description = operation.get("summary", operation.get("description", "No description provided."))
             logger.debug(f"Registering function: {function_name} - {function_description}")
+            input_schema = {
+                "type": "object",
+                "properties": {},
+                "required": [],
+                "additionalProperties": False
+            }
+            for param in operation.get("parameters", []):
+                param_name = param.get("name")
+                param_type = param.get("type", "string")
+                if param_type not in ["string", "integer", "boolean", "number"]:
+                    param_type = "string"
+                input_schema["properties"][param_name] = {
+                    "type": param_type,
+                    "description": param.get("description", f"{param.get('in', 'unknown')} parameter {param_name}")
+                }
+                if param.get("required", False):
+                    input_schema["required"].append(param_name)
             functions[function_name] = {
                 "name": function_name,
                 "description": function_description,
                 "path": path,
                 "method": method.upper(),
                 "operationId": operation.get("operationId"),
-                "original_name": raw_name
+                "original_name": raw_name,
+                "inputSchema": input_schema
             }
     logger.info(f"Discovered {len(functions)} functions from the OpenAPI specification.")
     logger.debug(f"Functions list: {list(functions.values())}")
@@ -86,7 +109,10 @@ def list_functions(*, env_key: str = "OPENAPI_SPEC_URL") -> str:
 
 @mcp.tool()
 def call_function(*, function_name: str, parameters: dict = None, env_key: str = "OPENAPI_SPEC_URL") -> str:
+    """Calls a function derived from the OpenAPI specification."""
     logger.debug(f"call_function invoked with function_name='{function_name}' and parameters={parameters}")
+    logger.debug(f"API_KEY: {os.getenv('API_KEY', '<not set>')[:5] + '...' if os.getenv('API_KEY') else '<not set>'}")
+    logger.debug(f"STRIP_PARAM: {os.getenv('STRIP_PARAM', '<not set>')}")
     if not function_name:
         logger.error("function_name is empty or None")
         return json.dumps({"error": "function_name is required"})
@@ -102,6 +128,7 @@ def call_function(*, function_name: str, parameters: dict = None, env_key: str =
     function_def = None
     paths = spec.get("paths", {})
     logger.debug(f"Paths for function lookup: {list(paths.keys())}")
+    prefix = get_tool_prefix()
     for path, path_item in paths.items():
         logger.debug(f"Checking path: {path}")
         for method, operation in path_item.items():
@@ -111,6 +138,8 @@ def call_function(*, function_name: str, parameters: dict = None, env_key: str =
                 continue
             raw_name = f"{method.upper()} {path}"
             current_function_name = normalize_tool_name(raw_name)
+            if prefix:
+                current_function_name = f"{prefix}{current_function_name}"
             logger.debug(f"Comparing {current_function_name} with {function_name}")
             if current_function_name == function_name:
                 function_def = {
@@ -126,38 +155,52 @@ def call_function(*, function_name: str, parameters: dict = None, env_key: str =
         logger.error(f"Function '{function_name}' not found in the OpenAPI specification.")
         return json.dumps({"error": f"Function '{function_name}' not found"})
     logger.debug(f"Function def found: {function_def}")
+
+    operation = function_def["operation"]
+    operation["method"] = function_def["method"]
+    headers = handle_auth(operation)
+    parameters = strip_parameters(parameters)
+    if function_def["method"] != "GET":
+        headers["Content-Type"] = "application/json"
+
     if not is_tool_whitelisted(function_def["path"]):
         logger.error(f"Access to function '{function_name}' is not allowed.")
         return json.dumps({"error": f"Access to function '{function_name}' is not allowed"})
 
-    # Build base URL using the utility function
     base_url = build_base_url(spec)
     if not base_url:
         logger.error("Failed to construct base URL from spec or SERVER_URL_OVERRIDE.")
         return json.dumps({"error": "No base URL defined in spec or SERVER_URL_OVERRIDE"})
 
     path = function_def["path"]
-    path = "/" + path.lstrip("/")
-    logger.debug(f"Normalized path: {path}")
-    api_url = base_url + path
-    logger.debug(f"Final API URL: {api_url}")
+    api_url = f"{base_url.rstrip('/')}/{path.lstrip('/')}"
     request_params = {}
     request_body = None
-    headers = {}
-    if function_def["method"] != "GET":
-        headers["Content-Type"] = "application/json"
-    headers.update(get_auth_headers(spec))
-    if parameters is None:
-        logger.debug("Parameters is None, using empty dict")
-        parameters = {}
-    logger.debug(f"Parameters received: {parameters}")
-    if parameters:
+
+    if isinstance(parameters, dict):
+        path_params_in_openapi = [
+            param["name"] for param in operation.get("parameters", []) if param.get("in") == "path"
+        ]
+        if path_params_in_openapi:
+            missing_required = [
+                param["name"] for param in operation.get("parameters", [])
+                if param.get("in") == "path" and param.get("required", False) and param["name"] not in parameters
+            ]
+            if missing_required:
+                logger.error(f"Missing required path parameters: {missing_required}")
+                return json.dumps({"error": f"Missing required path parameters: {missing_required}"})
+            for param_name in path_params_in_openapi:
+                if param_name in parameters:
+                    api_url = api_url.replace(f"{{{param_name}}}", str(parameters.pop(param_name)))
+                    logger.debug(f"Replaced path param {param_name} in URL: {api_url}")
         if function_def["method"] == "GET":
             request_params = parameters
-            logger.debug(f"Set request_params for GET: {request_params}")
         else:
             request_body = parameters
-            logger.debug(f"Set request_body: {request_body}")
+    else:
+        parameters = {}
+        logger.debug("No valid parameters provided, proceeding without params/body")
+
     logger.debug(f"Sending request - Method: {function_def['method']}, URL: {api_url}, Headers: {headers}, Params: {request_params}, Body: {request_body}")
     try:
         response = requests.request(
@@ -165,7 +208,7 @@ def call_function(*, function_name: str, parameters: dict = None, env_key: str =
             url=api_url,
             headers=headers,
             params=request_params if function_def["method"] == "GET" else None,
-            json=None if function_def["method"] == "GET" else request_body
+            json=request_body if function_def["method"] != "GET" else None
         )
         response.raise_for_status()
         logger.debug(f"API response received: {response.text}")
@@ -175,6 +218,7 @@ def call_function(*, function_name: str, parameters: dict = None, env_key: str =
         return json.dumps({"error": f"API request failed: {e}"})
 
 def run_simple_server():
+    """Runs the FastMCP server."""
     logger.debug("Starting run_simple_server")
     spec_url = os.environ.get("OPENAPI_SPEC_URL")
     if not spec_url:
