@@ -15,12 +15,12 @@ from mcp import types
 from mcp.server.lowlevel import Server
 from mcp.server.models import InitializationOptions
 from mcp.server.stdio import stdio_server
-from mcp_openapi_proxy.utils import setup_logging, normalize_tool_name, is_tool_whitelisted, fetch_openapi_spec, get_auth_headers, detect_response_type, build_base_url
+from mcp_openapi_proxy.utils import setup_logging, normalize_tool_name, is_tool_whitelisted, fetch_openapi_spec, build_base_url, handle_auth, strip_parameters, detect_response_type
 
 DEBUG = os.getenv("DEBUG", "").lower() in ("true", "1", "yes")
 logger = setup_logging(debug=DEBUG)
 
-tools: List[types.Tool] = []
+tools: List[types.Tool] = []  # Global tools list
 openapi_spec_data = None
 
 mcp = Server("OpenApiProxy-LowLevel")
@@ -31,6 +31,8 @@ async def dispatcher_handler(request: types.CallToolRequest) -> types.ServerResu
     try:
         function_name = request.params.name
         logger.debug(f"Dispatcher received CallToolRequest for function: {function_name}")
+        logger.debug(f"API_KEY: {os.getenv('API_KEY', '<not set>')[:5] + '...' if os.getenv('API_KEY') else '<not set>'}")
+        logger.debug(f"STRIP_PARAM: {os.getenv('STRIP_PARAM', '<not set>')}")
         tool = next((tool for tool in tools if tool.name == function_name), None)
         if not tool:
             logger.error(f"Unknown function requested: {function_name}")
@@ -40,7 +42,7 @@ async def dispatcher_handler(request: types.CallToolRequest) -> types.ServerResu
                 )
             )
         arguments = request.params.arguments or {}
-        logger.debug(f"Raw function arguments: {arguments}")
+        logger.debug(f"Raw arguments before processing: {arguments}")
 
         operation_details = lookup_operation_details(function_name, openapi_spec_data)
         if not operation_details:
@@ -50,11 +52,17 @@ async def dispatcher_handler(request: types.CallToolRequest) -> types.ServerResu
                     content=[types.TextContent(type="text", text=f"Could not find OpenAPI operation for function: {function_name}")]
                 )
             )
-        path = operation_details['path']
-        method = operation_details['method']
-        operation = operation_details['operation']
 
-        # Build base URL using the utility function
+        operation = operation_details['operation']
+        operation['method'] = operation_details['method']
+        headers = handle_auth(operation)
+        parameters = strip_parameters(arguments)
+        method = operation_details['method']
+        if method != "GET":
+            headers["Content-Type"] = "application/json"
+
+        path = operation_details['path']
+
         base_url = build_base_url(openapi_spec_data)
         if not base_url:
             logger.critical("Failed to construct base URL from spec or SERVER_URL_OVERRIDE.")
@@ -65,62 +73,54 @@ async def dispatcher_handler(request: types.CallToolRequest) -> types.ServerResu
             )
 
         api_url = f"{base_url.rstrip('/')}/{path.lstrip('/')}"
-        path_params = {}
-        query_params = {}
-        headers = {}
-        if method != "GET":
-            headers["Content-Type"] = "application/json"
-        headers.update(get_auth_headers(openapi_spec_data))
+        request_params = {}
         request_body = None
 
-        # Extract path and query parameters from arguments
-        if isinstance(arguments, dict):
-            params = arguments
-            if 'parameters' in arguments:
-                params = arguments['parameters']
-            if not isinstance(params, dict):
-                logger.error(f"Expected arguments or parameters to be a dict, got: {type(params)}")
-                return types.ServerResult(
-                    root=types.CallToolResult(
-                        content=[types.TextContent(type="text", text="Invalid parameters format")]
-                    )
-                )
-            # Handle path parameters
+        if isinstance(parameters, dict):
             path_params_in_openapi = [
-                param['name'] for param in operation.get('parameters', []) if param['in'] == 'path'
+                param['name'] for param in operation.get('parameters', []) if param.get('in') == 'path'
             ]
-            for param_name in path_params_in_openapi:
-                if param_name in params:
-                    path_params[param_name] = params.pop(param_name)
-                    api_url = api_url.replace(f"{{{param_name}}}", str(path_params[param_name]))
-            query_params = params
+            if path_params_in_openapi:
+                missing_required = [
+                    param['name'] for param in operation.get('parameters', [])
+                    if param.get('in') == 'path' and param.get('required', False) and param['name'] not in parameters
+                ]
+                if missing_required:
+                    logger.error(f"Missing required path parameters: {missing_required}")
+                    return types.ServerResult(
+                        root=types.CallToolResult(
+                            content=[types.TextContent(type="text", text=f"Missing required path parameters: {missing_required}")]
+                        )
+                    )
+                for param_name in path_params_in_openapi:
+                    if param_name in parameters:
+                        api_url = api_url.replace(f"{{{param_name}}}", str(parameters.pop(param_name)))
+                        logger.debug(f"Replaced path param {param_name} in URL: {api_url}")
+            if method == "GET":
+                request_params = parameters
+            else:
+                request_body = parameters
         else:
-            logger.debug("No valid arguments, proceeding without query params")
+            logger.debug("No valid parameters provided, proceeding without params/body")
 
-        logger.debug(f"API Request URL: {api_url}")
-        logger.debug(f"Request Method: {method}")
-        logger.debug(f"Path Parameters: {path_params}")
-        logger.debug(f"Query Parameters: {query_params}")
-        logger.debug(f"Request Headers: {headers}")
+        logger.debug(f"API Request - URL: {api_url}, Method: {method}")
+        logger.debug(f"Headers: {headers}")
+        logger.debug(f"Query Params: {request_params}")
+        logger.debug(f"Request Body: {request_body}")
 
         try:
             response = requests.request(
                 method=method,
                 url=api_url,
-                params=query_params if query_params else None,
                 headers=headers,
-                json=None if method == "GET" else request_body
+                params=request_params if method == "GET" else None,
+                json=request_body if method != "GET" else None
             )
             response.raise_for_status()
             response_text = response.text or "No response body"
-
-            # Detect response type using shared utility
             content, log_message = detect_response_type(response_text)
             logger.debug(log_message)
-
-            # Prepare content for MCP
             final_content = [content]
-
         except requests.exceptions.RequestException as e:
             logger.error(f"API request failed: {e}")
             return types.ServerResult(
@@ -141,12 +141,11 @@ async def dispatcher_handler(request: types.CallToolRequest) -> types.ServerResu
         logger.error(f"Unhandled exception in dispatcher_handler: {e}", exc_info=True)
         return types.ServerResult(
             root=types.CallToolResult(
-                content=[types.TextContent(type="text", text=str(e))]
+                content=[types.TextContent(type="text", text=f"Internal error: {str(e)}")]
             )
         )
 
 async def list_tools(request: types.ListToolsRequest) -> types.ServerResult:
-    """Handler for ListToolsRequest to list all registered functions (tools)."""
     logger.debug("Handling list_tools request - start")
     logger.debug(f"Tools list length: {len(tools)}")
     result = types.ServerResult(root=types.ListToolsResult(tools=tools))
@@ -155,26 +154,24 @@ async def list_tools(request: types.ListToolsRequest) -> types.ServerResult:
     return result
 
 def register_functions(spec: Dict) -> List[types.Tool]:
-    """Register functions (tools) dynamically based on the OpenAPI specification."""
+    """Register tools from OpenAPI spec, preserving across calls if already populated."""
     global tools
+    if tools:  # If tools already exist, don’t reset unless spec changes
+        logger.debug("Tools already registered, skipping re-registration")
+        return tools
     tools = []
-
     if not spec:
         logger.error("OpenAPI spec is None or empty.")
         return tools
-
     if 'paths' not in spec:
         logger.error("No 'paths' key in OpenAPI spec.")
         return tools
-
     logger.debug(f"Spec paths available: {list(spec['paths'].keys())}")
     filtered_paths = {path: item for path, item in spec['paths'].items() if is_tool_whitelisted(path)}
     logger.debug(f"Filtered paths: {list(filtered_paths.keys())}")
-
     if not filtered_paths:
         logger.warning("No whitelisted paths found in OpenAPI spec after filtering.")
         return tools
-
     for path, path_item in filtered_paths.items():
         if not path_item:
             logger.debug(f"Empty path item for {path}")
@@ -187,8 +184,6 @@ def register_functions(spec: Dict) -> List[types.Tool]:
                 raw_name = f"{method.upper()} {path}"
                 function_name = normalize_tool_name(raw_name)
                 description = operation.get('summary', operation.get('description', 'No description available'))
-
-                # Build inputSchema from operation parameters
                 input_schema = {
                     "type": "object",
                     "properties": {},
@@ -208,7 +203,6 @@ def register_functions(spec: Dict) -> List[types.Tool]:
                         }
                         if param.get('required', False):
                             input_schema['required'].append(param_name)
-
                 tool = types.Tool(
                     name=function_name,
                     description=description,
@@ -218,12 +212,10 @@ def register_functions(spec: Dict) -> List[types.Tool]:
                 logger.debug(f"Registered function: {function_name} ({method.upper()} {path}) with inputSchema: {json.dumps(input_schema)}")
             except Exception as e:
                 logger.error(f"Error registering function for {method.upper()} {path}: {e}", exc_info=True)
-
     logger.info(f"Registered {len(tools)} functions from OpenAPI spec.")
     return tools
 
 def lookup_operation_details(function_name: str, spec: Dict) -> Dict or None:
-    """Lookup OpenAPI operation details (path, method, operation) based on function name."""
     if not spec or 'paths' not in spec:
         return None
     for path, path_item in spec['paths'].items():
@@ -237,7 +229,6 @@ def lookup_operation_details(function_name: str, spec: Dict) -> Dict or None:
     return None
 
 async def start_server():
-    """Start the Low-Level MCP server."""
     logger.debug("Starting Low-Level MCP server...")
     async with stdio_server() as (read_stream, write_stream):
         await mcp.run(
@@ -251,31 +242,26 @@ async def start_server():
         )
 
 def run_server():
-    """Run the Low-Level Any OpenAPI server."""
     global openapi_spec_data
     try:
         openapi_url = os.getenv('OPENAPI_SPEC_URL')
         if not openapi_url:
             logger.critical("OPENAPI_SPEC_URL environment variable is required but not set.")
             sys.exit(1)
-
         openapi_spec_data = fetch_openapi_spec(openapi_url)
         if not openapi_spec_data:
             logger.critical("Failed to fetch or parse OpenAPI specification from OPENAPI_SPEC_URL.")
             sys.exit(1)
         logger.info("OpenAPI specification fetched successfully.")
         logger.debug(f"Full OpenAPI spec: {json.dumps(openapi_spec_data, indent=2)}")
-
         register_functions(openapi_spec_data)
         logger.debug(f"Tools after registration: {[tool.name for tool in tools]}")
         if not tools:
             logger.critical("No valid tools registered. Shutting down.")
             sys.exit(1)
-
         mcp.request_handlers[types.ListToolsRequest] = list_tools
         mcp.request_handlers[types.CallToolRequest] = dispatcher_handler
         logger.debug("Handlers registered.")
-
         asyncio.run(start_server())
     except KeyboardInterrupt:
         logger.debug("MCP server shutdown initiated by user.")
