@@ -1,7 +1,7 @@
 """
 Utility functions for mcp_openapi_proxy, including logging setup,
 OpenAPI fetching, name normalization, whitelist filtering, auth handling,
-and response type detection.
+parameter stripping, and response type detection.
 """
 
 import os
@@ -20,15 +20,6 @@ load_dotenv()
 OPENAPI_SPEC_URL = os.getenv("OPENAPI_SPEC_URL")
 
 def setup_logging(debug: bool = False) -> logging.Logger:
-    """
-    Configures logging for the application, directing all output to stderr.
-
-    Args:
-        debug (bool): If True, sets log level to DEBUG; otherwise, INFO.
-
-    Returns:
-        logging.Logger: Configured logger instance.
-    """
     logger = logging.getLogger(__name__)
     logger.setLevel(logging.DEBUG if debug else logging.INFO)
     logger.propagate = False
@@ -52,26 +43,11 @@ logger.debug(f"OpenAPI Spec URL: {OPENAPI_SPEC_URL}")
 logger.debug("utils.py initialized")
 
 def redact_api_key(key: str) -> str:
-    """Redacts an API key for secure logging."""
     if not key or len(key) <= 4:
         return "<not set>"
     return f"{key[:2]}{'*' * (len(key) - 4)}{key[-2:]}"
 
 def normalize_tool_name(name: str) -> str:
-    """
-    Normalizes tool names into a clean function name without parameters, 
-    ensuring compliance with ^[a-zA-Z0-9_-]+$.
-
-    For example:
-    - 'GET /sessions/{sessionId}/messages/{messageUUID}' becomes 'get_sessions_messages'
-    - 'POST /users.create' becomes 'post_users_create'
-
-    Args:
-        name (str): Raw method and path, e.g., 'GET /sessions/{sessionId}/messages/{messageUUID}'.
-
-    Returns:
-        str: Normalized tool name without special characters.
-    """
     if not name or not isinstance(name, str):
         logger.warning(f"Invalid tool name input: {name}. Defaulting to 'unknown_tool'.")
         return "unknown_tool"
@@ -96,30 +72,19 @@ def normalize_tool_name(name: str) -> str:
             continue
         func_name += f"_{part.replace('.', '_')}"
     func_name = re.sub(r"[^a-zA-Z0-9_-]", "_", func_name)
-    func_name = re.sub(r"_+", "_", func_name).strip("_").lower()  # Force lowercase
+    func_name = re.sub(r"_+", "_", func_name).strip("_").lower()
     if len(func_name) > 64:
         func_name = func_name[:64]
     logger.debug(f"Normalized tool name from '{name}' to '{func_name}'")
     return func_name or "unknown_tool"
 
 def get_tool_prefix() -> str:
-    """Retrieves tool name prefix from environment, ensuring it ends with an underscore."""
     prefix = os.getenv("TOOL_NAME_PREFIX", "")
     if prefix and not prefix.endswith("_"):
         prefix += "_"
     return prefix
 
 def is_tool_whitelisted(endpoint: str) -> bool:
-    """
-    Checks if an endpoint matches any partial path in TOOL_WHITELIST.
-    Supports exact matches, prefix matches, and partial path matches with variables.
-
-    Args:
-        endpoint (str): The endpoint path from the OpenAPI spec (e.g., '/sessions/{sessionId}/messages').
-
-    Returns:
-        bool: True if the endpoint matches any whitelist item partially or exactly, False otherwise.
-    """
     whitelist = os.getenv("TOOL_WHITELIST", "")
     logger.debug(f"Checking whitelist - endpoint: {endpoint}, TOOL_WHITELIST: {whitelist}")
     if not whitelist:
@@ -157,8 +122,6 @@ def is_tool_whitelisted(endpoint: str) -> bool:
     return False
 
 def fetch_openapi_spec(spec_url: str) -> dict:
-    """Fetches and parses OpenAPI specification from a URL or file, supporting JSON and YAML formats."""
-    logger = logging.getLogger(__name__)
     try:
         if spec_url.startswith("file://"):
             spec_path = spec_url.replace("file://", "")
@@ -166,7 +129,10 @@ def fetch_openapi_spec(spec_url: str) -> dict:
                 content = f.read()
             logger.debug(f"Read local OpenAPI spec from {spec_path}")
         else:
-            response = requests.get(spec_url)
+            response = requests.get(spec_url, timeout=10)
+            if response.status_code in [401, 403]:
+                logger.debug(f"Spec {spec_url} requires auth (status {response.status_code})—skipping")
+                return None
             response.raise_for_status()
             content = response.text
             logger.debug(f"Fetched OpenAPI spec from {spec_url}")
@@ -175,7 +141,7 @@ def fetch_openapi_spec(spec_url: str) -> dict:
             logger.debug(f"Parsed YAML OpenAPI spec from {spec_url}")
         else:
             spec = json.loads(content)
-            logger.debug(f"Parsed JSON OpenAPI spec from {spec_url}")
+            logger.debug(f"Parsed JSON OpenAPI spec from {spec_url} (no suffix assumed JSON)")
         return spec
     except (requests.exceptions.RequestException, FileNotFoundError) as e:
         logger.error(f"Error fetching OpenAPI spec from {spec_url}: {e}")
@@ -184,50 +150,36 @@ def fetch_openapi_spec(spec_url: str) -> dict:
         logger.error(f"Error parsing OpenAPI spec from {spec_url}: {e}")
         return None
 
-def get_auth_headers(spec: dict, api_key_env: str = "API_KEY") -> dict:
+def handle_auth(operation: dict) -> dict:
     """
-    Constructs authorization headers based on spec and environment variables.
-
-    Args:
-        spec (dict): OpenAPI specification.
-        api_key_env (str): Environment variable name for the API key (default: "API_KEY").
-
-    Returns:
-        dict: Headers dictionary with Authorization set appropriately.
+    Handles authentication for API requests.
+    - API_KEY: Sets Bearer header.
+    Returns: headers
     """
+    api_key = os.getenv("API_KEY")
     headers = {}
-    auth_token = os.getenv(api_key_env)
-    if not auth_token:
-        logger.debug(f"No {api_key_env} set, skipping auth headers.")
-        return headers
-    auth_type_override = os.getenv("API_AUTH_TYPE")
-    if auth_type_override:
-        headers["Authorization"] = f"{auth_type_override} {auth_token}"
-        logger.debug(f"Using API_AUTH_TYPE override: Authorization: {auth_type_override} {redact_api_key(auth_token)}")
-        return headers
-    security_defs = spec.get('securityDefinitions', {})
-    for name, definition in security_defs.items():
-        if definition.get('type') == 'apiKey' and definition.get('in') == 'header' and definition.get('name') == 'Authorization':
-            desc = definition.get('description', '')
-            match = re.search(r'(\w+(?:-\w+)*)\s+<token>', desc)
-            if match:
-                prefix = match.group(1)
-                headers["Authorization"] = f"{prefix} {auth_token}"
-                logger.debug(f"Using apiKey with prefix from spec description: Authorization: {prefix} {redact_api_key(auth_token)}")
-            else:
-                headers["Authorization"] = auth_token
-                logger.debug(f"Using raw apiKey auth from spec: Authorization: {redact_api_key(auth_token)}")
-            return headers
-        elif definition.get('type') == 'oauth2':
-            headers["Authorization"] = f"Bearer {auth_token}"
-            logger.debug(f"Using Bearer auth from spec: Authorization: Bearer {redact_api_key(auth_token)}")
-            return headers
-    headers["Authorization"] = auth_token
-    logger.warning(f"No clear auth type in spec, using raw API key: Authorization: {redact_api_key(auth_token)}")
+    if api_key:
+        headers["Authorization"] = f"Bearer {api_key}"
+        logger.debug(f"Using API_KEY as Bearer: {redact_api_key(api_key)}")
+    logger.debug(f"Headers after auth: {headers}")
     return headers
 
+def strip_parameters(parameters: dict = None) -> dict:
+    """
+    Strips specified parameter from parameters if STRIP_PARAM is set.
+    Returns: updated_parameters
+    """
+    if parameters is None:
+        parameters = {}
+    logger.debug(f"Raw parameters before stripping: {parameters}")
+    strip_param = os.getenv("STRIP_PARAM")
+    if strip_param and isinstance(parameters, dict) and strip_param in parameters:
+        del parameters[strip_param]
+        logger.debug(f"Stripped param '{strip_param}' from parameters")
+    logger.debug(f"Parameters after stripping: {parameters}")
+    return parameters
+
 def map_schema_to_tools(schema: dict) -> list:
-    """Maps a schema to a list of MCP tools."""
     tools = []
     classes = schema.get("classes", [])
     for entry in classes:
@@ -244,15 +196,6 @@ def map_schema_to_tools(schema: dict) -> list:
     return tools
 
 def detect_response_type(response_text: str) -> tuple[types.TextContent, str]:
-    """
-    Detect the response type (JSON or text) and return the appropriate MCP content object.
-
-    Args:
-        response_text (str): The raw response text from the HTTP request.
-
-    Returns:
-        Tuple: (content object, log message)
-    """
     try:
         json_data = json.loads(response_text)
         structured_text = {"text": response_text}
@@ -264,15 +207,6 @@ def detect_response_type(response_text: str) -> tuple[types.TextContent, str]:
     return content, log_message
 
 def build_base_url(spec: dict) -> str:
-    """
-    Constructs the base URL for API requests, prioritizing SERVER_URL_OVERRIDE.
-
-    Args:
-        spec (dict): OpenAPI specification containing servers or host information.
-
-    Returns:
-        str: The constructed base URL, normalized to remove trailing slashes, or the spec URL if placeholders are present.
-    """
     override = os.getenv("SERVER_URL_OVERRIDE", "").strip()
     if override:
         urls = override.split()
